@@ -28,13 +28,6 @@
 #include "uvm_ioctl.h"
 #include "nv_uvm_types.h"
 
-#define UVM_TEST_SKIP_MIGRATE_VMA                        UVM_TEST_IOCTL_BASE(103)
-typedef struct
-{
-    NvBool skip;                                         // In
-    NV_STATUS rmStatus;                                  // Out
-} UVM_TEST_SKIP_MIGRATE_VMA_PARAMS;
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -47,7 +40,6 @@ typedef struct
 {
     // In params
     NvProcessorUuid gpu_uuid;
-    NvU32           swizz_id;
     // Out params
     NvU64           ref_count NV_ALIGN_BYTES(8);
     NV_STATUS       rmStatus;
@@ -198,7 +190,8 @@ typedef struct
     NvU64                           va_range_end                     NV_ALIGN_BYTES(8); // Out, inclusive
     NvU32                           read_duplication;                                   // Out (UVM_TEST_READ_DUPLICATION_POLICY)
     NvProcessorUuid                 preferred_location;                                 // Out
-    NvProcessorUuid                 accessed_by[UVM_MAX_PROCESSORS];                    // Out
+    NvS32                           preferred_cpu_nid;                                  // Out
+    NvProcessorUuid                 accessed_by[UVM_MAX_PROCESSORS_V2];                 // Out
     NvU32                           accessed_by_count;                                  // Out
     NvU32                           type;                                               // Out (UVM_TEST_VA_RANGE_TYPE)
     union
@@ -354,20 +347,30 @@ typedef enum
     UVM_TEST_CHANNEL_STRESS_MODE_NOOP_PUSH = 0,
     UVM_TEST_CHANNEL_STRESS_MODE_UPDATE_CHANNELS,
     UVM_TEST_CHANNEL_STRESS_MODE_STREAM,
+    UVM_TEST_CHANNEL_STRESS_MODE_KEY_ROTATION,
 } UVM_TEST_CHANNEL_STRESS_MODE;
 
+typedef enum
+{
+    UVM_TEST_CHANNEL_STRESS_KEY_ROTATION_OPERATION_CPU_TO_GPU,
+    UVM_TEST_CHANNEL_STRESS_KEY_ROTATION_OPERATION_GPU_TO_CPU,
+    UVM_TEST_CHANNEL_STRESS_KEY_ROTATION_OPERATION_ROTATE,
+} UVM_TEST_CHANNEL_STRESS_KEY_ROTATION_OPERATION;
+
 #define UVM_TEST_CHANNEL_STRESS                          UVM_TEST_IOCTL_BASE(15)
 typedef struct
 {
-    NvU32     mode;                   // In
+    NvU32     mode;                   // In, one of UVM_TEST_CHANNEL_STRESS_MODE
 
     // Number of iterations:
     //   mode == NOOP_PUSH: number of noop pushes
     //   mode == UPDATE_CHANNELS: number of updates
     //   mode == STREAM: number of iterations per stream
+    //   mode == ROTATION: number of operations
     NvU32     iterations;
 
-    NvU32     num_streams;            // In, used only for mode == UVM_TEST_CHANNEL_STRESS_MODE_STREAM
+    NvU32     num_streams;            // In, used only if mode == STREAM
+    NvU32     key_rotation_operation; // In, used only if mode == ROTATION
     NvU32     seed;                   // In
     NvU32     verbose;                // In
     NV_STATUS rmStatus;               // Out
@@ -511,7 +514,12 @@ typedef struct
 typedef struct
 {
     // In params
-    UvmEventEntry entry; // contains only NvUxx types
+    union
+    {
+        UvmEventEntry_V1 entry_v1; // contains only NvUxx types
+        UvmEventEntry_V2 entry_v2; // contains only NvUxx types
+    };
+    NvU32 version;
     NvU32 count;
 
     // Out param
@@ -568,6 +576,22 @@ typedef struct
 // user_pages_allocation_retry_force_count, but the injection point simulates
 // driver metadata allocation failure.
 //
+// cpu_chunk_allocation_target_id and cpu_chunk_allocation_actual_id are used
+// to control the NUMA node IDs for CPU chunk allocations, specifically for
+// testing overlapping CPU chunk allocations.
+//
+// Currently, uvm_api_migrate() does not pass the preferred CPU NUMA node to for
+// managed memory so it is not possible to request a specific node.
+// cpu_chunk_allocation_target_id is used to request the allocation be made on
+// specific node. On the other hand, cpu_chunk_allocation_actual_id is the node
+// on which the allocation will actually be made.
+//
+// The two parameters can be used to force a CPU chunk allocation to overlap a
+// previously allocated chunk.
+//
+// Please note that even when specifying cpu_cpu_allocation_actual_id, the
+// kernel may end up allocating on a different node.
+//
 // Error returns:
 // NV_ERR_INVALID_ADDRESS
 //  - lookup_address doesn't match a UVM range
@@ -578,6 +602,8 @@ typedef struct
     NvU32     page_table_allocation_retry_force_count;  // In
     NvU32     user_pages_allocation_retry_force_count;  // In
     NvU32     cpu_chunk_allocation_size_mask;           // In
+    NvS32     cpu_chunk_allocation_target_id;           // In
+    NvS32     cpu_chunk_allocation_actual_id;           // In
     NvU32     cpu_pages_allocation_error_count;         // In
     NvBool    eviction_error;                           // In
     NvBool    populate_error;                           // In
@@ -608,31 +634,35 @@ typedef struct
 
     // Array of processors which have a resident copy of the page containing
     // lookup_address.
-    NvProcessorUuid                 resident_on[UVM_MAX_PROCESSORS];                    // Out
+    NvProcessorUuid                 resident_on[UVM_MAX_PROCESSORS_V2];                 // Out
     NvU32                           resident_on_count;                                  // Out
 
+    // If the memory is resident on the CPU, the NUMA node on which the page
+    // is resident. Otherwise, -1.
+    NvS32                           resident_nid;                                       // Out
+
     // The size of the physical allocation backing lookup_address. Only the
     // system-page-sized portion of this allocation which contains
     // lookup_address is guaranteed to be resident on the corresponding
     // processor.
-    NvU32                           resident_physical_size[UVM_MAX_PROCESSORS];         // Out
+    NvU32                           resident_physical_size[UVM_MAX_PROCESSORS_V2];      // Out
 
     // The physical address of the physical allocation backing lookup_address.
-    NvU64                           resident_physical_address[UVM_MAX_PROCESSORS] NV_ALIGN_BYTES(8); // Out
+    NvU64                           resident_physical_address[UVM_MAX_PROCESSORS_V2] NV_ALIGN_BYTES(8); // Out
 
     // Array of processors which have a virtual mapping covering lookup_address.
-    NvProcessorUuid                 mapped_on[UVM_MAX_PROCESSORS];                      // Out
-    NvU32                           mapping_type[UVM_MAX_PROCESSORS];                   // Out
-    NvU64                           mapping_physical_address[UVM_MAX_PROCESSORS] NV_ALIGN_BYTES(8); // Out
+    NvProcessorUuid                 mapped_on[UVM_MAX_PROCESSORS_V2];                   // Out
+    NvU32                           mapping_type[UVM_MAX_PROCESSORS_V2];                // Out
+    NvU64                           mapping_physical_address[UVM_MAX_PROCESSORS_V2] NV_ALIGN_BYTES(8); // Out
     NvU32                           mapped_on_count;                                    // Out
 
     // The size of the virtual mapping covering lookup_address on each
     // mapped_on processor.
-    NvU32                           page_size[UVM_MAX_PROCESSORS];                      // Out
+    NvU32                           page_size[UVM_MAX_PROCESSORS_V2];                   // Out
 
     // Array of processors which have physical memory populated that would back
     // lookup_address if it was resident.
-    NvProcessorUuid                 populated_on[UVM_MAX_PROCESSORS];                   // Out
+    NvProcessorUuid                 populated_on[UVM_MAX_PROCESSORS_V2];                // Out
     NvU32                           populated_on_count;                                 // Out
 
     NV_STATUS rmStatus;                                                                 // Out
@@ -1175,19 +1205,6 @@ typedef struct
     NV_STATUS                       rmStatus;                                           // Out
 } UVM_TEST_PMM_QUERY_PMA_STATS_PARAMS;
 
-#define UVM_TEST_NUMA_GET_CLOSEST_CPU_NODE_TO_GPU        UVM_TEST_IOCTL_BASE(77)
-typedef struct
-{
-    NvProcessorUuid                 gpu_uuid;                                           // In
-    NvHandle                        client;                                             // In
-    NvHandle                        smc_part_ref;                                       // In
-
-    // On kernels with NUMA support, this entry contains the closest CPU NUMA
-    // node to this GPU. Otherwise, the value will be -1.
-    NvS32                           node_id;                                            // Out
-    NV_STATUS                       rmStatus;                                           // Out
-} UVM_TEST_NUMA_GET_CLOSEST_CPU_NODE_TO_GPU_PARAMS;
-
 // Test whether the bottom halves have run on the correct CPUs based on the
 // NUMA node locality of the GPU.
 //
@@ -1203,8 +1220,6 @@ typedef struct
 typedef struct
 {
     NvProcessorUuid                 gpu_uuid;                                           // In
-    NvHandle                        client;                                             // In
-    NvHandle                        smc_part_ref;                                       // In
 
     NV_STATUS                       rmStatus;                                           // Out
 } UVM_TEST_NUMA_CHECK_AFFINITY_PARAMS;
@@ -1441,6 +1456,28 @@ typedef struct
 {
     NV_STATUS rmStatus;                                  // Out
 } UVM_TEST_CPU_CHUNK_API_PARAMS;
+
+#define UVM_TEST_FORCE_CPU_TO_CPU_COPY_WITH_CE          UVM_TEST_IOCTL_BASE(101)
+typedef struct
+{
+    NvBool force_copy_with_ce;                          // In
+    NV_STATUS rmStatus;                                 // Out
+} UVM_TEST_FORCE_CPU_TO_CPU_COPY_WITH_CE_PARAMS;
+
+#define UVM_TEST_VA_SPACE_ALLOW_MOVABLE_ALLOCATIONS     UVM_TEST_IOCTL_BASE(102)
+typedef struct
+{
+    NvBool allow_movable;                               // In
+    NV_STATUS rmStatus;                                 // Out
+} UVM_TEST_VA_SPACE_ALLOW_MOVABLE_ALLOCATIONS_PARAMS;
+
+#define UVM_TEST_SKIP_MIGRATE_VMA                        UVM_TEST_IOCTL_BASE(103)
+typedef struct
+{
+    NvBool skip;                                         // In
+    NV_STATUS rmStatus;                                  // Out
+} UVM_TEST_SKIP_MIGRATE_VMA_PARAMS;
+
 #ifdef __cplusplus
 }
 #endif
