@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1999-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1999-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -52,6 +52,8 @@
 
 #include <ctrl/ctrl00fd.h>
 
+#include <ctrl/ctrl00e0.h>
+
 #define NV_CTL_DEVICE_ONLY(nv)                 \
 {                                              \
     if (((nv)->flags & NV_FLAG_CONTROL) == 0)  \
@@ -86,6 +88,9 @@ static NV_STATUS RmGetDeviceFd(NVOS54_PARAMETERS *pApi, NvS32 *pFd,
         case NV00FD_CTRL_CMD_ATTACH_GPU:
             paramSize = sizeof(NV00FD_CTRL_ATTACH_GPU_PARAMS);
             break;
+        case NV00E0_CTRL_CMD_EXPORT_MEM:
+            paramSize = sizeof(NV00E0_CTRL_EXPORT_MEM_PARAMS);
+            break;
         default:
             return NV_OK;
     }
@@ -106,6 +111,15 @@ static NV_STATUS RmGetDeviceFd(NVOS54_PARAMETERS *pApi, NvS32 *pFd,
                  *pFd = (NvS32)pAttachGpuParams->devDescriptor;
             }
             break;
+        case NV00E0_CTRL_CMD_EXPORT_MEM:
+            {
+                NV00E0_CTRL_EXPORT_MEM_PARAMS *pExportMemParams = pKernelParams;
+
+                // If hParent is client, no need to reference device.
+                *pSkipDeviceRef = (pExportMemParams->hParent == pApi->hClient);
+                *pFd = (NvS32)pExportMemParams->devDescriptor;
+            }
+            break;
         default:
             NV_ASSERT(0);
             break;
@@ -153,25 +167,11 @@ static void RmCreateOsDescriptor(NVOS32_PARAMETERS *pApi, API_SECURITY_INFO secI
     }
     else if (rmStatus == NV_ERR_INVALID_ADDRESS)
     {
-        rmStatus = os_lookup_user_io_memory(pDescriptor, pageCount,
-                &pPteArray, &pPageArray);
+        rmStatus = os_lookup_user_io_memory(pDescriptor, pageCount, &pPteArray);
         if (rmStatus == NV_OK)
         {
-            if (pPageArray != NULL)
-            {
-                pApi->data.AllocOsDesc.descriptor = (NvP64)(NvUPtr)pPageArray;
-                pApi->data.AllocOsDesc.descriptorType = NVOS32_DESCRIPTOR_TYPE_OS_PAGE_ARRAY;
-            }
-            else if (pPteArray != NULL)
-            {
-                pApi->data.AllocOsDesc.descriptor = (NvP64)(NvUPtr)pPteArray;
-                pApi->data.AllocOsDesc.descriptorType = NVOS32_DESCRIPTOR_TYPE_OS_IO_MEMORY;
-            }
-            else
-            {
-                NV_ASSERT_FAILED("unknown memory import type");
-                rmStatus = NV_ERR_NOT_SUPPORTED;
-            }
+            pApi->data.AllocOsDesc.descriptor = (NvP64)(NvUPtr)pPteArray;
+            pApi->data.AllocOsDesc.descriptorType = NVOS32_DESCRIPTOR_TYPE_OS_IO_MEMORY;
         }
     }
     if (rmStatus != NV_OK)
